/*
=================
VolumetricFog.h
-includes the function, initialiseVolumetricFogExtension, used to initialise the OpenGL extensions
-Note: to enable volumetric fog your video card must support the "GL_EXT_fog_coord" extension
=================
*/
#ifndef VolumetricFog_H
#define VolumetricFog_H

#include <GL/glext.h> //list of OpenGL extensions
#include <string>

/* value taken from the glext.h header file 
   -PFNGLFOGCOORDFEXTPROC takes a floating point value 
   -initialise glFogCoordfEXT to NULL */
PFNGLFOGCOORDFEXTPROC glFogCoordfEXT = NULL;

/* specifies the fog colour */
GLfloat	fog_colour[4] = {0.0, 0.5, 0.9f, 1.0f};

/*
=================
initialiseVolumetricFogExtension
-function enables volumetric fog
=================
*/
void initialiseVolumetricFogExtension()
{
	/*  assign glFogCoordfEXT the address of the specific video card's volumetric fog extension 
	    -this routine enables volumetric fog 
		Note: this will generate an error if your video card doesn't support volumetric fog*/
	glFogCoordfEXT = (PFNGLFOGCOORDFEXTPROC)wglGetProcAddress("glFogCoordfEXT");
}

#endif
//EOF